var _ = require('lodash');
var React = require('react');

var ConfigItem = require('./ConfigItem.jsx');
var ConfigGroupDescription = require('./ConfigGroupDescription.jsx');
var ConfigTestWidget = require('./ConfigTestWidget.jsx');
var ItemStore = require('../services/ItemStore');

var ConfigGroup = React.createClass({
  isAtLeastOneItemVisible: function() {
    return _.some(this.props.group.items, function(item) {
      if (!_.isEmpty(item)) {
        return ItemStore.isItemVisible(item);
      }
    }.bind(this));
  },

  handleItemChange: function(itemName, value, data) {
    if (this.props.handleChange) {
      this.props.handleChange(itemName, value, data);
    }
  },

  render: function() {
    if (!this.isAtLeastOneItemVisible()) {
      return <span></span>;
    }

    var title = null;
    if (this.props.group.title) {
      title = (
        <div className="col-xs-12">
          <div className="page-header">
            <div className="spacer"></div>
            <h3 className="group-title">{this.props.group.title}</h3>
          </div>
        </div>
      );
    }

    var description = null;
    if (this.props.group.description) {
      description = (
        <ConfigGroupDescription text={this.props.group.description} />
      );
    }

    var items = [];
    _.each(this.props.group.items, function(item) {
      if (!_.isEmpty(item)) {
        items.push(
          <ConfigItem
            key={item.name}
            item={item}
            visible={ItemStore.isItemVisible(item)}
            handleChange={this.handleItemChange}
            testUrl={this.props.testUrl} />
        );
      }
    }.bind(this));

    var testWidget = null;
    if (_.get(this.props.group, 'test_proc.command', null) && this.props.group.test_proc.command.length > 0) {
      testWidget = (
        <div className="form-group col-xs-12">
          <ConfigTestWidget
            command={this.props.group.test_proc.command}
            timeout={this.props.group.test_proc.timeout}
            argFields={this.props.group.test_proc.arg_fields}
            testUrl={this.props.testUrl} />
        </div>
      );
    }

    return (
      <div className="config-group row clearfix">
        {title}
        {description}
        {items}
        {testWidget}
      </div>
    );
  }
});

module.exports = ConfigGroup;
