var _ = require('lodash');
var classNames = require('classnames');
var React = require('react');

var ConfigItemTitle = require('./ConfigItemTitle.jsx');
var ConfigItemDescription = require('./ConfigItemDescription.jsx');
var ConfigGroupDescription = require('./ConfigGroupDescription.jsx');
var ConfigItemHelpText = require('./ConfigItemHelpText.jsx');
var ConfigTestWidget = require('./ConfigTestWidget.jsx');

var fields = {
  'text': React.createFactory(require('./ConfigFieldText.jsx')),
  'password': React.createFactory(require('./ConfigFieldPassword.jsx')),
  'file': React.createFactory(require('./ConfigFieldFile.jsx')),
  'bool': React.createFactory(require('./ConfigFieldCheckbox.jsx')),
  'select_one': React.createFactory(require('./ConfigFieldSelectOne.jsx')),
  'select_many': React.createFactory(require('./ConfigFieldSelectMany.jsx')),
  'label': React.createFactory(require('./ConfigFieldLabel.jsx')),
  'heading': React.createFactory(require('./ConfigFieldHeading.jsx'))
};

var ConfigItem = React.createClass({
  handleItemChange: function(name, value, data) {
    if (this.props.handleChange) {
      this.props.handleChange(name, value, data);
    }
  },

  render: function() {
    var title = null, description = null, field = null, testWidget = null;

    function getItemTestWidget(item, testUrl) {
      if (_.get(item, 'test_proc.command', null) && item.test_proc.command.length > 0) {
        return (
          <div style={{marginTop: '10px'}}>
            <ConfigTestWidget
              command={item.test_proc.command}
              timeout={item.test_proc.timeout}
              argFields={item.test_proc.arg_fields}
              testUrl={testUrl} />
          </div>
        );
      }
      return null;
    }

    if (this.props.item.type === 'label') {
      title = (
        <ConfigItemTitle title={this.props.item.title} />
      );
      if (_.get(this.props.item, 'description', null) && this.props.item.description.length > 0) {
        description = (
          <ConfigItemDescription text={this.props.item.description} />
        );
      }
    } else if (this.props.item.type === 'heading') {
      title = (
        <div className="page-header clearfix">
          <div className="spacer"></div>
          <h3 className="group-title">{this.props.item.title}</h3>
        </div>
      );
      if (_.get(this.props.item, 'description', null) && this.props.item.description.length > 0) {
        description = (
          <div className="row">
            <ConfigGroupDescription text={this.props.item.description} />
          </div>
        );
      }
    } else {
      title = (
        <ConfigItemTitle
          title={this.props.item.title}
          required={this.props.item.required === true}
          recommended={this.props.item.recommended === true} />
      );
      if (_.get(this.props.item, 'description', null) && this.props.item.description.length > 0) {
        description = (
          <ConfigItemDescription text={this.props.item.description} />
        );
      }
      testWidget = getItemTestWidget(
        this.props.item,
        this.props.testUrl
      );
    }

    var props = {
      title: title,
      name: this.props.item.name,
      value: this.props.item.value,
      default: this.props.item.default,
      readOnly: this.props.item.readonly === true,
      items: this.props.item.items || [],
      handleChange: this.handleItemChange
    };

    if (this.props.item.type in fields) {
      field = fields[this.props.item.type](props);
    } else if (this.props.item.type) {
      // unknown field type
      return null;
    }

    var helpText = null;
    if (_.get(this.props.item, 'help_text', null) && this.props.item.help_text.length > 0) {
      helpText = <ConfigItemHelpText text={this.props.item.help_text} />;
    }

    var classes = classNames('form-group col-xs-12', {
      'col-sm-6': (
        this.props.item.affix === 'left' ||
        this.props.item.affix === 'right'
      ),
      'hidden': !this.props.visible
    });

    return (
      <div className={classes}>
        {field}
        {description}
        {helpText}
        {testWidget}
      </div>
    );
  }
});

module.exports = ConfigItem;
