var gulp = require('gulp'),
  gutil = require('gulp-util'),
  source = require('vinyl-source-stream'),
  buffer = require('vinyl-buffer'),
  rename = require('gulp-rename'),
  sourcemaps = require('gulp-sourcemaps'),
  browserify = require('browserify'),
  watchify = require('watchify'),
  reactify = require('reactify'),
  less = require('gulp-less'),
  _ = require('lodash');

// Directory where static files are found. Don't forget the slash at the end.
var path = {
  DEST: './build',
  JS: {
    OUT: 'main.js',
    ENTRYPOINT: './preview/main.js'
  },
  CSS: {
    OUT: 'main.css',
    ENTRYPOINT: './preview/styles.less'
  }
};

// Browserify bundler, configured for reactify with sources having a .jsx extension
var customOpts = {
  entries: [path.JS.ENTRYPOINT],
  transform: [reactify],
  extensions: ['.jsx'],
  debug: true
};
var opts = _.assign({}, watchify.args, customOpts);
var b = browserify(opts);
b.on('log', gutil.log);

b.transform('brfs');

function bundle() {
  return b.bundle()
    .on('log', gutil.log)
    .on('error', gutil.log.bind(gutil, 'Error'))
    .pipe(source(path.JS.OUT))
    .pipe(buffer())
    .pipe(sourcemaps.init({loadMaps: true})) // loads map from browserify file
    .pipe(sourcemaps.write('./')) // writes .map file
    .pipe(gulp.dest(path.DEST));
}

// Build JavaScript using Browserify
gulp.task('js', bundle);

// Build CSS
gulp.task('css', function(){
  return gulp.src(path.CSS.ENTRYPOINT)
  .pipe(sourcemaps.init())
  .pipe(less())
  .pipe(sourcemaps.write())
  .pipe(rename(path.CSS.OUT))
  .pipe(gulp.dest(path.DEST));
});

gulp.task('jswatch', function() {
  watchify(b)
  .on('update', bundle);
});

gulp.task('csswatch', function () {
  gulp.watch('**/*.less', ['css']);
});

gulp.task('watch', ['jswatch', 'csswatch', 'js', 'css']);
gulp.task('default', ['js', 'css']);
