var _ = require('lodash');

var ItemStore = {
  setGroups: function(groups) {
    this.items = _.without(_.flatten(_.map(groups, function(group) {
      if (!_.isEmpty(group)) {
        return _.map(_.get(group, 'items', []), function(item) {
          if (!_.isEmpty(item)) {
            return item;
          }
        });
      }
      return null;
    })), null);
  },

  getItem: function(itemName) {
    var item = null;
    _.some(this.items, function(otherItem) {
      if (otherItem.name === itemName) {
        item = otherItem;
        return true;
      } else {
        return _.some(otherItem.items, function(otherChildItem) {
          if (otherChildItem.name === itemName) {
            item = otherChildItem;
            return true;
          }
          return false;
        });
      }
    });
    return item;
  },

  isItemVisible: function(item) {
    if (item.hidden) {
      return false;
    }
    return this.isItemEnabled(item);
  },

  isItemEnabled: function(item) {
    var checkWhen = function(i, v, ne) {
      var val = _.get(i, 'value', null);
      if (ne) {
        return val && val.length > 0 ? i.value !== v : i.default !== v;
      } else {
        return val && val.length > 0 ? i.value === v : i.default === v;
      }
    };

    if (_.get(item, 'when', null) && item.when.length > 0) {
      var when = item.when.split('=', 2);
      if (when.length === 2) {
        var opNe = false,
          itemName = when[0],
          itemValue = when[1];
        if (itemName.substr(itemName.length - 1) === '!') {
          opNe = true;
          itemName = itemName.substr(0, itemName.length - 1);
        }
        var theItem = ItemStore.getItem(itemName);
        if (theItem !== null) {
          return checkWhen(theItem, itemValue, opNe);
        }
      }
    }

    return true;
  }
};

module.exports = ItemStore;
